﻿-- Daily Global Check
-- by Jadya
-- EU-Well of Eternity

local addonName, addonTable = ...

local max = math.max

local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local ldbicon = ldb and LibStub("LibDBIcon-1.0", true)
addonTable.ldbicon = ldbicon

local L = addonTable.L -- localization
local C = addonTable.C

-- string colors
local AddColor = addonTable.AddColor

local addontitle = "|cff00AAFFDaily Global Check|r"
local addontitle_short = "|cff00AAFFDGC|r"
local dgcicon = "Interface\\icons\\INV_Sigil_Freya"

-- constants
local ZONE = 1
local NAME = 2
local PREF = 3
local SUFF = 4
local MAPDATA = 5
local MAPDEFAULT = 6
local QUESTTYPE = 7 -- nil == daily
local MAPICON = 8 -- nil == plugin icon
local SHOWFUNC = 9
local COMPLETEDFUNC = 10
local PLUGIN = 11
local HIDDEN = 12 -- hidden in the editframe quests pool
local OVERLAY = 13

addonTable.maxlists = 30

function DailyGlobalCheck:GetVersion()
 return C.version
end

function DailyGlobalCheck:GetPluginLoadDelayMax()
 return C.plugin_load_delay_max
end

local mapiconSizeL = 21 -- current map icons
local mapiconSizeS = 13 -- adjacent map icons

-- saved variables
DailyGlobalCheck_Options = {}
DGC_CustomLists = {}
DailyGlobalCheck_CharData = {}
DailyGlobalCheck_PluginData = {}

-- ext
local chars = DailyGlobalCheck.CharFunctions
local pages = DailyGlobalCheck.PagesFunctions
local options = DailyGlobalCheck.OptFunctions
local constructors = DailyGlobalCheck.Constructors
local lists = DailyGlobalCheck.ListFunctions
local maps = DailyGlobalCheck.MapFunctions
local quest = DailyGlobalCheck.QuestFunctions
---------------- editframe interface -------------
DailyGlobalCheck.EditPanel = constructors:Panel(UIParent, "DGC_EditPanel", 700, 800, true)
local editframe = DailyGlobalCheck.EditPanel
------------------------------------------------------

local charData = DailyGlobalCheck_CharData
local pName = GetUnitName("player")
local pRealm = GetRealmName()
addonTable.pID = pName.."-"..pRealm
local pID = addonTable.pID
-- pools
DailyGlobalCheck.Lists = {}
--
DailyGlobalCheck.player_faction = UnitFactionGroup("player")
DailyGlobalCheck.selectedpID = pID
---
-- quests data pool
DailyGlobalCheck.QuestsData = {}
local questsdata = DailyGlobalCheck.QuestsData

local SelectedList
local DGCTTmenu = {}
local DGCMMmenu = {}

function DailyGlobalCheck:GetSelectedList()
 return SelectedList
end

-- needed to split fixed lists (plugins) and custom lists (saved variables)
function DailyGlobalCheck:foreachLists(func, active)
 for k,v in pairs(DailyGlobalCheck.Lists) do
  if active == false or lists:isActive(v) then
   func(k,v)
  end
 end
 
 if not DGC_CustomLists.Lists then return end
 
 for k,v in pairs(DGC_CustomLists.Lists) do
  if active == false or lists:isActive(v) then
   func(k,v)
  end
 end
end

-- needed to split fixed quest data (plugins) and custom quest data (saved variables)
function DailyGlobalCheck:foreachQData(func)
 for k,v in pairs(DailyGlobalCheck.QuestsData) do
  func(k,v)
 end
 
 if not DGC_CustomLists.QuestsData then return end
 
 for k,v in pairs(DGC_CustomLists.QuestsData) do
  func(k,v)
 end
end

function DailyGlobalCheck:getQData(questID)
 return DailyGlobalCheck.QuestsData[questID] or DGC_CustomLists.QuestsData[questID]
end
--

-------------
local template = {[ZONE] = "", [NAME] = L["namemissing"], [PREF] = "", [SUFF] = "", [QUESTTYPE] = "D"}
local function get(tab, i)
 return tab[i] or template[i]
end
------------------------------------------------------------------------------
local function GetCoord(questID)

 local function beautify(coord)
  local s = tostring(math.floor(coord * 10) / 10)
  local i = strfind(s, "%.")
  if i then
   if i < 2 then s = "0"..s end
  else
   s = s..".0"
  end
  if coord < 10 then s = "0"..s end
  return s
 end

 local data = DailyGlobalCheck:getQData(questID)
 local v = data[MAPDATA]
 local default = data[MAPDEFAULT]
 return v and AddColor(beautify(v[default][1]).."  "..beautify(v[default][2]), "COORD_COLOR") or ""
end
------------------------------------------------------------------------------
local function isquestcompleted(questID, mainCharOnly, notSelectedPlugin)

 local list = notSelectedPlugin or SelectedList
 if not mainCharOnly and DailyGlobalCheck.selectedpID ~= pID and list.MultiCharsEnabled then
  return chars:isquestcompleted(questID, DailyGlobalCheck.selectedpID)
 end
 
 if questsdata[questID] and questsdata[questID][COMPLETEDFUNC] then
  return questsdata[questID][COMPLETEDFUNC](questID)
 elseif type(questID) ~= "number" then
  local result, cs, reset = quest:isquestcompleted(questID)
  return result, cs, reset
 else
  return IsQuestFlaggedCompleted(questID)
 end
end
addonTable.isquestcompleted = isquestcompleted
------------------------------------------------------------------------------
local function isshown(questID, list)
 local qData = DailyGlobalCheck:getQData(questID)

 if not qData then return false end
  -- fire the show func if present
  
 return not qData[SHOWFUNC] or qData[SHOWFUNC](questID)
end

function DailyGlobalCheck:RefreshTitle()
 local f = DGC_Mainframe
 
 if not f then return end

 local plugintitle = SelectedList and AddColor(SelectedList["Title"],"LIST_COLOR") or ""
 if SelectedList then
  local suff = pID ~= DailyGlobalCheck.selectedpID and AddColor(L["opt_characters_short"], "LIGHT_BLUE") or ""
  local title = SelectedList.MultiCharsEnabled and DailyGlobalCheck.selectedpID..suff or ""
  f.title:SetText(title)
  -- pages
  f.page:SetText(pages:GetString(SelectedList))
 end
 f.Ptitle:SetText(plugintitle)
end
---------------------------- TomTom Menu Frame -------------------------------------
local function setTomTomWaypoint(...)
 TomTom:AddMFWaypoint(...)
 local t = select(5, ...)
 print(L["s_tomtomset"].." ("..AddColor(t.title, "LIGHT_BLUE")..")")
end

function DailyGlobalCheck:ShowTomTomMenu()
 if not SelectedList then return end
 DGCMenuFrame:SetPoint("TOP",DGC_Mainframe.TomTombutton,"BOTTOM",0,-5)
 EasyMenu(DGCTTmenu, DGCMenuFrame, DGCMenuFrame, 0 , 0, "MENU")
end

local function GenerateTomTomMenu()

 if not TomTom or not DGCMenuFrame or not SelectedList then return end

  local function CreateSubMenu(q)
   local submenu = {}
   for index,questID in pairs(q) do
    local data = DailyGlobalCheck:getQData(questID)
    if index > 1 and data and isshown(questID) and data[MAPDATA] then
    table.insert(submenu, { text = get(data, NAME).." - "..AddColor(get(data, ZONE),"GRAY").." "..GetCoord(questID),
                            disabled = isquestcompleted(questID) or not data[MAPDATA],
                            notCheckable = 1,
                            keepShownOnClick = true,
                            func = function(self)
                               if data and data[MAPDATA] then
                                 setTomTomWaypoint(data[MAPDEFAULT], nil,
                                            data[MAPDATA][data[MAPDEFAULT]][1] / 100,
                                            data[MAPDATA][data[MAPDEFAULT]][2] / 100,
                                          { title = get(data, NAME)})
                               end
                              end
                              })
     end
    end
    return submenu
   end

  wipe(DGCTTmenu)
  table.insert(DGCTTmenu, {text = AddColor("- TomTom Waypoints -","GOLD"),notCheckable = 1, isTitle = true})
  table.insert(DGCTTmenu, {text = AddColor("Click either a single quest or a group","LIGHT_BLUE"),notCheckable = 1, isTitle = true})
  table.insert(DGCTTmenu, {text = AddColor("to set TomTom waypoints","LIGHT_BLUE"),notCheckable = 1, isTitle = true})
  table.insert(DGCTTmenu, {text = "",notCheckable = 1, isTitle = true})

  table.foreach(SelectedList["Order"], function(k,tab)
   local pref = "|T"..SelectedList["Icon"]..":12|t"
   table.foreach(tab, function(header,quests)
    if table.getn(quests) > 1 then
    table.insert(DGCTTmenu, { text = quests[1] ~= "" and pref..quests[1] or pref..L["notitle"],
                              keepShownOnClick = true,
                              notCheckable = 1,
                       func = function(self)
                               table.foreach(quests, function(index, questID)
							    if index > 1 then
 								 local data = DailyGlobalCheck:getQData(questID)
                                  if data and data[MAPDATA] and not isquestcompleted(questID) and isshown(questID) then
                                   TomTom:AddMFWaypoint(data[MAPDEFAULT], nil,
                                              data[MAPDATA][data[MAPDEFAULT]][1] / 100,
                                              data[MAPDATA][data[MAPDEFAULT]][2] / 100,
                                            { title = get(data, NAME) } )
                                   print(L["s_tomtomset"].." ("..AddColor(get(data, NAME),"LIGHT_BLUE")..")")
                                 end
								end
                               end)
                              end,
                       hasArrow = true,
		               menuList = CreateSubMenu(quests)})
	end
   end)
  end)
  table.insert(DGCTTmenu, {text = "", isTitle = true, notCheckable = 1})
  table.insert(DGCTTmenu, {text = CLOSE, notCheckable = 1})
end

---------------------------- Generate Line (used by drawtooltip and /dgc print) -------------------------------------
local function GenerateLine(questID, cstring)
 local s1 = ""
 local data = DailyGlobalCheck:getQData(questID)
	
 if not data then return L["questdatamissing"] end

 -- quest type
 if DailyGlobalCheck_Options["show_questtype"] then
  s1 = s1..AddColor("["..get(data, QUESTTYPE).."]","LIGHT_BLUE") 
 end
 -- prefix
 if DailyGlobalCheck_Options["show_prefix"] then
  s1 = s1..AddColor(get(data, PREF), "WHITE")
 end
 -- zone name
 if DailyGlobalCheck_Options["show_zone"] then
  s1 = s1..AddColor(get(data, ZONE),"GRAY")
 end
 -- quest name
 s1 = s1.." "..AddColor(get(data, NAME),"NAME_COLOR")
 -- suffix
 if DailyGlobalCheck_Options["show_suffix"] then
  s1 = s1..AddColor(get(data, SUFF).." ", "WHITE")
 end
 -- coordinates
 if DailyGlobalCheck_Options["show_coordinates"] then
  s1 = s1.." "..AddColor(GetCoord(questID),"COORD_COLOR")
 end
	
 return s1
end
------------------------------------------ Draw Tooltip --------------------------------------------------------
local function DrawTooltip(tt)
 if not SelectedList then
  if lists:Count() == 0 then
   tt:AddDoubleLine(AddColor("No active lists","LIGHT_RED")," ")
  else
   tt:AddDoubleLine(AddColor(L["nolist"],"LIGHT_RED")," ")
  end
 else
  local questsorder = SelectedList["Order"]
  local index = pages:Curr()
  
  if SelectedList["OnShow"] then SelectedList["OnShow"]() end
  options:OnDraw()
  
  if not questsorder[index] then return end
  
  local s1,s2
  for k,v in pairs(questsorder[index]) do
   if v[1] ~= "" then
    tt:AddDoubleLine(AddColor(v[1],"GOLD")," ")
   end
   for i = 2,#v do
    local questID = v[i]
	 if isshown(questID) then
	  local completed, cstring, reset = isquestcompleted(questID, istooltip)
	  if not DailyGlobalCheck_Options["hide_completed"] or (not completed or cstring or reset) then
	   if cstring == nil then cstring = completed and COMPLETE or INCOMPLETE end
       s2 = AddColor(cstring, completed and "LIGHT_GREEN" or "LIGHT_RED")
       s1 = GenerateLine(questID)
       tt:AddDoubleLine(s1,s2,questID)
	  end
	 end
    end
    tt:AddDoubleLine(" "," ")
   end
  end
  
 tt:AddLine(pages:GetString(SelectedList))
 tt:AddLine(AddColor(L["lclicktoopen"],"LIGHT_BLUE"))
 tt:AddLine(AddColor(L["rclicktoopen"],"LIGHT_BLUE"))
 tt:Show()
end

------------------------------------------ Draw Frame (fill button) ----------------------------------------------------
local columns_w = {}

local function getBtn(i)
 local result = DGC_Mainframe.buttons[i] or constructors:QuestButton(i, DGC_Mainframe.buttons_container)
 
 if not DGC_Mainframe.buttons[i] then
  DGC_Mainframe.buttons[i] = result
 end
 
 return result
end
 
local function adjustColumnsSize(f)
 local edt = DGC_Mainframe.editmode
 columns_w[ZONE] = edt and 0 or max(columns_w[ZONE] or 0, f.zone:GetStringWidth())
 columns_w[NAME] = max(columns_w[NAME] or 0, f.name:GetStringWidth())
 columns_w[PREF] = edt and 0 or max(columns_w[PREF] or 0, f.prefix:GetStringWidth())
 columns_w[SUFF] = edt and 0 or max(columns_w[SUFF] or 0, f.suffix:GetStringWidth())
 columns_w[5]    = edt and 0 or max(columns_w[5]    or 0, f.coords:GetStringWidth())
 columns_w[6]    = edt and 0 or max(columns_w[6]    or 0, f.completed:GetStringWidth())
end

local function fillButtonText(i, s1, color1, s2, color2)
 
 local f = getBtn(i)

 f.questtype:Hide()
 f.prefix:SetText(AddColor(s1 or "", color1))
 f.zone:SetText("")
 f.name:SetText("")
 f.suffix:SetText("")
 f.coords:SetText("")
 f.completed:SetText(AddColor(s2 or "", color2))

 f.prefix:SetWidth(f.prefix:GetStringWidth())
 f.completed:SetWidth(f.completed:GetStringWidth())
 --adjustColumnsSize(f)
 return f
end

local function fillButtonData(i, questID, cstring)

 local f = getBtn(i)

 local data = DailyGlobalCheck:getQData(questID)
 local o = DailyGlobalCheck_Options
 local left_spacer = ""
 
 if o["show_questtype"] then
  if data[QUESTTYPE] == "Q" then
   f.questtype:SetTexCoord(0.125, 0.256, 0.125, 0.256)
  elseif data[7] == "W" then
   f.questtype:SetTexCoord(0.625, 0.75, 0.125, 0.256)
  else
   f.questtype:SetTexCoord(0.375, 0.5, 0.125, 0.256)
  end
  left_spacer = "  "
  f.questtype:Show()
 else
  f.questtype:Hide()
 end
 
 f.prefix:SetText(left_spacer..(o["show_prefix"] and get(data, PREF) or ""))
 f.zone:SetText(o["show_zone"] and (AddColor(get(data, ZONE),"GRAY").." ") or "")
 f.name:SetText(AddColor(get(data, NAME),"NAME_COLOR"))
 f.suffix:SetText(o["show_suffix"] and get(data, SUFF) or "")
 f.coords:SetText(o["show_coordinates"] and (" "..AddColor(GetCoord(questID),"COORD_COLOR")) or "")
 f.completed:SetText(cstring)

 local info = quest:GetOverlayInfo(questID)
 if info then
  f.overlayInfo = info
  f.overlay:Show()
 else
  f.overlay:Hide()
 end
 
 adjustColumnsSize(f)
 return f
end

------------------------------------------ Draw Frame --------------------------------------------------------
local function DrawFrame()
 
 local f = DGC_Mainframe
 
 local button = 1
 wipe(columns_w)
 
 if not SelectedList then
  if lists:Count() == 0 then
   fillButtonText(1, "No Lists Available", "LIGHT_RED")
  else
   fillButtonText(1, L["nolist"], "LIGHT_RED")
  end
  return
 end
 
 ---
 local function fillButton(t, ...)
  local btn
  if t == "data" then
   btn = fillButtonData(button, ...)
   btn.noresize = nil
  elseif t == "text" then
   btn = fillButtonText(button, ...)
   btn.noresize = true
  else
   return
  end
  btn:Show()
  button = button + 1
 end
 ---
 
 local questsorder = SelectedList["Order"]
 local index = pages:Curr()

 if SelectedList["OnShow"] then SelectedList["OnShow"]() end
 options:OnDraw()

 if not questsorder[index] then return end
  
 local s1,s2
 for k,v in pairs(questsorder[index]) do
  if v[1] ~= "" then
   fillButton("text", v[1], "GOLD")
  end

  for i = 2,#v do
   local questID = v[i]
   if isshown(questID) then
    local completed, cstring, reset = isquestcompleted(questID, istooltip)
    if not DailyGlobalCheck_Options["hide_completed"] or (not completed or cstring or reset) then
     if cstring == nil then cstring = completed and COMPLETE or INCOMPLETE end
	 fillButton("data", questID, AddColor(cstring, completed and "LIGHT_GREEN" or "LIGHT_RED"))
    end
   end
  end
  fillButton("text", " ", "GOLD")
 end

 -- hide the remaining buttons
 f:HideQuestButtons(button - 1)

 f.dgcicon:SetTexture(SelectedList["Icon"] or dgcicon)

 -- create some space for the buttons
 local offset = C.pluginbtnsize * 5

 -- editframe lock
 if not f.editmode then
  local w = 0

  for i = 1,6 do
   w = w + (columns_w[i] or 0)
  end

  f:SetSize(max(300, w + 30), button * 12 + offset)

  for i = 1, button - 1 do
   if not f.buttons[i].noresize then
    f.buttons[i].zone:SetWidth(columns_w[ZONE] or 0)
    f.buttons[i].name:SetWidth(columns_w[NAME] or 0)
    f.buttons[i].prefix:SetWidth(columns_w[PREF] or 0)
    f.buttons[i].suffix:SetWidth(columns_w[SUFF] or 0)
    f.buttons[i].coords:SetWidth(columns_w[5] or 0)
    f.buttons[i].completed:SetWidth(columns_w[6] or 0)
   end
  end
  local title = DGC_Mainframe.Ptitle
  local w = title:GetWidth()
  title:SetPoint("TOPLEFT", DGC_Mainframe, "TOPLEFT", (DGC_Mainframe:GetWidth() - 70) / 2 - w / 2, -15)
 end 
end
---------------------------- Chat Output ---------------------------------------------
local function ChatOutput()
 if not SelectedList or not SelectedList["Order"][1] then return end

 local s1,s2
 for k,v in pairs(SelectedList["Order"][1]) do
 if v[1] ~= "" then
  print(AddColor(v[1],"GOLD"))
 end
 for i,questID in pairs(v) do
        if i > 1 then
	    if isshown(questID) then
			local completed, cstring = isquestcompleted(questID)
			if cstring == nil then cstring = completed and COMPLETE or INCOMPLETE end
            s2 = AddColor(cstring,completed and "LIGHT_GREEN" or "LIGHT_RED")
            s1 = GenerateLine(questID)
            print(s1.." "..s2)
		end
     end
  end
 end
end
---------------------------- set frames data -----------------------------------------
function DailyGlobalCheck:PushData(v, cfunc, plugin, refresh)
 for qID,qData in pairs(v) do
  if not questsdata[qID] then
  --if not DailyGlobalCheck:getQData(qID) then
   questsdata[qID] = qData
   questsdata[qID][COMPLETEDFUNC] = cfunc
   questsdata[qID][PLUGIN] = plugin
   maps:resolveCoords(questsdata[qID])
  end
 end

 if refresh and SelectedList and SelectedList["Title"] == plugin then
  DailyGlobalCheck:Refresh()
 end
end

function DailyGlobalCheck:SetFramesData()
 -- still here for compatibility reasons
end

function DailyGlobalCheck:OpenList(v)
 local list = type(v) == "string" and select(2, lists:Find(v)) or v
 
 if not v then
  SelectedList = nil
  DailyGlobalCheck:Refresh()
  GenerateTomTomMenu()
  -- refresh the map only if it has to draw only the selected list
  if DailyGlobalCheck_Options["show_mapicons_all"] then
   addonTable.forcemapupdate()
  end
  DailyGlobalCheck_PluginData["last_selected"] = nil
  return
 end
 
 if not lists:isList(list) then return end
 
 SelectedList = list
 DailyGlobalCheck_PluginData["last_selected"] = list["Title"]
 GenerateTomTomMenu()
 -- refresh the map only if it has to draw only the selected list
 if DailyGlobalCheck_Options["show_mapicons_all"] then
  addonTable.forcemapupdate()
 end
 DailyGlobalCheck:Refresh()
 if DGCMenuFrame then DGCMenuFrame:Hide() end
 if DGC_Mainframe then
  DailyGlobalCheck:RefreshTitle()
  DGC_Mainframe:SelectListButton(SelectedList["Title"])
  DrawFrame()
 end
end
---------------------------------------------------------------------
local function GenerateMinimapMenu()
 if not DGCMMmenu then return end

  local function CreateListsMenu()
   local submenu = {}
    DailyGlobalCheck:foreachLists(function(k,v)
    table.insert(submenu, { text = v["Title"],
                            --disabled = false,
                            notCheckable = 1,
                            keepShownOnClick = true,
                            func = function()
                               DailyGlobalCheck:OpenList(v)
                              end
                              })
    end)
    return submenu
   end

  DGCMMmenu = {}
  table.insert(DGCMMmenu, {text = AddColor(L["availablelists"],"GOLD"),notCheckable = 1, hasArrow = true, menuList = CreateListsMenu()})
  
  if TomTom and DGCTTmenu then
   table.insert(DGCMMmenu, {text = AddColor("TomTom","GOLD"),notCheckable = 1, hasArrow = true, menuList = DGCTTmenu})
  end
  
  table.insert(DGCMMmenu, {text = "", isTitle = true, notCheckable = 1})

  table.insert(DGCMMmenu, {text = L["prevpage"], notCheckable = 1, func = function() 
   if pages:Move(SelectedList, -1) and DGC_Mainframe then
    DGC_Mainframe:SetAnchor("BOTTOMLEFT")
    DailyGlobalCheck:Refresh(SelectedList)
   end
  end})
  
  table.insert(DGCMMmenu, {text = L["nextpage"], notCheckable = 1, func = function() 
   if pages:Move(SelectedList, 1) and DGC_Mainframe then
    DGC_Mainframe:SetAnchor("BOTTOMLEFT")
    DailyGlobalCheck:Refresh(SelectedList)
   end
  end})
--  table.insert(DGCTTmenu, {text = "",notCheckable = 1, isTitle = true})
  table.insert(DGCMMmenu, {text = "", isTitle = true, notCheckable = 1})
  table.insert(DGCMMmenu, {text = CLOSE, notCheckable = 1})
  EasyMenu(DGCMMmenu, DGCMenuFrame, "cursor", -100 , 0, "MENU")
end

-- Initialize Mainframe
function DGCInit(arg)

 if arg == "print" then
  print(AddColor("Daily Global Check","LIGHT_BLUE"))
  local last_pID = DailyGlobalCheck.selectedpID
  DailyGlobalCheck.selectedpID = addonTable.pID
  ChatOutput()
  DailyGlobalCheck.selectedpID = last_pID
  print(AddColor("---","LIGHT_BLUE"))
  return
 end

 -- constructor moved to contr_mainframe.lua
 if not DGC_Mainframe then constructors:MainFrame() end
 local f = DGC_Mainframe
 editframe:Deactivate()
 
 local a1, _, a2, x, y = unpack(DailyGlobalCheck_Options["frame_position"] or {})
 if not a1 then
  f:SetPoint("CENTER")
 else
  f:SetPoint(a1, UIParent, a2, x, y)
 end
 
 if f:IsVisible() then
  f:Hide()
  if DGCMenuFrame then DGCMenuFrame:Hide() end
  collectgarbage()
 else
  DrawFrame()
  if SelectedList and SelectedList["Title"] then
   f:SelectListButton(SelectedList["Title"])
  end
  f:Show()
 end
end

--- world map section
local mapID
local active_buttons = 1
DailyGlobalCheck_mapframe = CreateFrame("Frame")
DailyGlobalCheck_mapframe:SetParent(WorldMapButton)
DailyGlobalCheck_mapframe:SetAllPoints()
DailyGlobalCheck_mapframe.framespool = {}
CreateFrame("GameTooltip","DailyGlobalCheck_maptooltip", nil, "GameTooltipTemplate")
DailyGlobalCheck_maptooltip:SetFrameStrata("TOOLTIP")
DailyGlobalCheck_maptooltip:SetScale(0.8)

local function createmapbutton()
 local f = CreateFrame("FRAME")
 f:SetParent(DailyGlobalCheck_mapframe)
 f:SetPoint("CENTER",DailyGlobalCheck_mapframe,"CENTER")
 f.data = {}
 f:SetScript("OnMouseUp", function(_, mousebutton)
   if IsControlKeyDown() and mousebutton == "RightButton" then
    local qID, list = f.data[1], f.data[2]

	if not list or not qID or not list["Data"][qID] then return end
    local qData = list["Data"][qID]
	
    local x = qData[MAPDATA][qData[MAPDEFAULT]][1] / 100
	local y = qData[MAPDATA][qData[MAPDEFAULT]][2] / 100
	setTomTomWaypoint(qData[MAPDEFAULT], nil, x, y, { title = get(qData, NAME)})
   else
    WorldMapButton_OnClick(WorldMapButton, mousebutton)
   end
  end)
 
 f:SetScript("OnLeave", function()
  DailyGlobalCheck_maptooltip:Hide()
 end)
 f.tex = f:CreateTexture()
 f.tex:SetAllPoints()
 f:SetFrameStrata("HIGH")
 table.insert(DailyGlobalCheck_mapframe.framespool, f)
 return f
end

local function addMapIcons(list)
 if not list then return end
 
  for _,t in pairs(list["Order"]) do
  for _,page in pairs(t) do
  for i,qID in pairs(page) do
  if i > 1 then
     if not isquestcompleted(qID, true, list) and isshown(qID, list) then
	  local v = DailyGlobalCheck:getQData(qID)
      if v[MAPDATA] and v[MAPDATA][mapID]  then
       -- pick up an unused frame or create a new one
       local f = DailyGlobalCheck_mapframe.framespool[active_buttons] ~= nil and DailyGlobalCheck_mapframe.framespool[active_buttons] or createmapbutton()
       local x = v[MAPDATA][mapID][1]
       local y = v[MAPDATA][mapID][2]
       f:SetWidth (v[MAPDEFAULT] == mapID and mapiconSizeL or mapiconSizeS)
       f:SetHeight(v[MAPDEFAULT] == mapID and mapiconSizeL or mapiconSizeS)
       f:SetPoint("CENTER",DailyGlobalCheck_mapframe,"TOPLEFT",
         -- I simply treat coordinates as a percentage of the width/height of the map frame
              DailyGlobalCheck_mapframe:GetWidth()  * x / 100,
             -DailyGlobalCheck_mapframe:GetHeight() * y / 100)
       f.data[1] = qID
	   f.data[2] = list
	   f.qData = v
	   f:SetScript("OnEnter", function(self)
	    if not self.qData then return end
        DailyGlobalCheck_maptooltip:SetOwner(f,"ANCHOR_BOTTOM")
        DailyGlobalCheck_maptooltip:ClearLines()
        DailyGlobalCheck_maptooltip:AddLine(addontitle)
        DailyGlobalCheck_maptooltip:AddLine(AddColor(list["Title"],"LIGHT_BLUE"))
        DailyGlobalCheck_maptooltip:AddLine(get(self.qData, PREF).." "..get(self.qData, NAME))
        DailyGlobalCheck_maptooltip:AddLine(get(self.qData, SUFF))
        DailyGlobalCheck_maptooltip:Show()
       end)
	   local _, plugin = lists:Find(v[PLUGIN])
       f.tex:SetTexture(v[MAPICON] or ((plugin and plugin["Icon"]) or dgcicon),1)
       f:Show()
       active_buttons = active_buttons + 1
      end
     end
	end
	end
	end
    end
end

local function DGCworldmapupdate()
  DailyGlobalCheck_mapframe:Hide()
  DailyGlobalCheck_maptooltip:Hide()
  if not DailyGlobalCheck_Options["show_mapicons"] or GetCurrentMapContinent() == WORLDMAP_COSMIC_ID 
     or GetNumDungeonMapLevels() ~= 0 then --or not WorldMapButton:IsVisible() then
   return
  end

  if mapID ~= GetCurrentMapAreaID() then
   for _,v in pairs(DailyGlobalCheck_mapframe.framespool) do
    v:Hide() 
   end

   active_buttons = 1
   mapID = GetCurrentMapAreaID()
   if not DailyGlobalCheck_Options["show_mapicons_all"] then
    DailyGlobalCheck:foreachLists(function(_,v)
     addMapIcons(v)
    end)
   else
    addMapIcons(SelectedList)
   end
  end

  if active_buttons > 1 then
   DailyGlobalCheck_mapframe:Show()
  end
end

-- resizing the map through those buttons does not fire the world_map_update event,
-- even if that changes the mapID to the player's current map, so I hook them
WorldMapFrameSizeDownButton:HookScript("OnClick", DGCworldmapupdate)
WorldMapFrameSizeUpButton:HookScript("OnClick", DGCworldmapupdate)
--- world map section end

function DailyGlobalCheck:Refresh(list)
 if not list or SelectedList == list then
  if DGC_Mainframe then
   DailyGlobalCheck:RefreshTitle()
   DGC_Mainframe:UpdateArrows()
   DrawFrame()
  end
 end
 editframe:Update()
end

addonTable.forcemapupdate = function()
 mapID = -1
 DGCworldmapupdate()
end

WorldMapButton:HookScript("OnShow", addonTable.forcemapupdate)

local ldbset = false
local eventframe = CreateFrame("FRAME","DGCEventFrame")
eventframe:RegisterEvent("WORLD_MAP_UPDATE")
eventframe:RegisterEvent("PLAYER_LOGIN")
function eventframe.WORLD_MAP_UPDATE(...)
 if WorldMapFrame:IsVisible() then
  DGCworldmapupdate()
 end
end

function eventframe.PLAYER_LOGIN(...)
  if DailyGlobalCheck_Options == nil then DailyGlobalCheck_Options = {} end
  
  local O = DailyGlobalCheck_Options

  if O["show_questtype"]   == nil then O["show_questtype"]   = false end
  if O["show_coordinates"] == nil then O["show_coordinates"] = true end
  if O["show_zone"]        == nil then O["show_zone"]        = false end
  if O["show_prefix"]      == nil then O["show_prefix"]      = false end
  if O["show_suffix"]      == nil then O["show_suffix"]      = false end
  if O["show_mapicons"]    == nil then O["show_mapicons"]    = true end
  if O["show_mapicons_all"]== nil then O["show_mapicons_all"]= false end
  if O["show_questlogbtn"] == nil then O["show_questlogbtn"] = true end
  if O["chars_blacklist"]  == nil then O["chars_blacklist"]  = {} end
  if O["hide_completed"]   == nil then O["hide_completed"]   = false end
  if O["frame_position"]   == nil then O["frame_position"]   = {"CENTER"} end

  if not DGC_CustomLists.Lists then DGC_CustomLists.Lists = {} end
  if not DGC_CustomLists.QuestsData then DGC_CustomLists.QuestsData = {} end

  -- v1.2 --> v1.3 conversion
  if O["DGC_Version"] and O["DGC_Version"] < 1003 then
   O["last_selected"] = nil
   
   for k,v in pairs(DGC_CustomLists) do
    if lists:isList(v) then
	 for qID,QuestsData in pairs(v["Data"]) do
      DGC_CustomLists.QuestsData[qID] = QuestsData
	 end
	 v["Data"] = {}
     table.insert(DGC_CustomLists.Lists, v)
	 DGC_CustomLists[k] = nil
	end
   end
  end
  -- future conversions v1.3 --> x  here
  --

  -- delete not existing lists from the character list settings saved var
  lists:UpdatePluginDataSVar()
  
  DGCMenuFrame = CreateFrame("Frame", "DGCMenuFrame", UIParent, "UIDropDownMenuTemplate")
  
  -- saved selected list
  local lastsel = DailyGlobalCheck_PluginData["last_selected"]
  if lastsel then
   DailyGlobalCheck:OpenList(lastsel)
  else
   local list = lists:FindFirst()
   DailyGlobalCheck:OpenList(list and list["Title"] or nil)
  end

  if O["minimap_icon"] == nil then -- show minimap icon
    O["minimap_icon"] = {
        hide = false,
        minimapPos = 220,
    }
  end
  O["DGC_Version"] = C.version

  if ldb and not ldbset then
        local DailyGlobalCheck_broker = ldb:NewDataObject("Daily Global Check", {
	        type = "data source",
	        icon = dgcicon,
	        label = "Daily Global Check",
	        OnClick = function(self,button)
			 if button == "LeftButton" then
               DGCInit()
			 elseif button == "RightButton" then
			  GenerateMinimapMenu()
			 end
	        end,
	        OnTooltipShow = function(tooltip)
			 if lists:isList(SelectedList) then
			  tooltip:ClearLines()
			  tooltip:AddLine(addontitle..AddColor(" ("..SelectedList["Title"]..")","LIGHT_BLUE"))
		      DrawTooltip(tooltip)
			 end
	    end,
        })
        if ldbicon then
            ldbicon:Register("DailyGlobalCheck_broker", DailyGlobalCheck_broker, O["minimap_icon"])
        end
   ldbset = true
 end
end

eventframe:SetScript("OnEvent", function(self, event, ...) eventframe[event](...) end)

-- "Add quest to Custom List" menubutton in the QuestMap frame
local function hookToggleDropDownMenu(level, value, dropDownFrame, anchorName, xOffset, yOffset, menuList, button, autoHideDelay)
 if dropDownFrame ~= QuestMapQuestOptionsDropDown or not DailyGlobalCheck_Options["show_questlogbtn"] then
  return
 end 

 local questLogIndex = GetQuestLogIndexByID(dropDownFrame.questID)
 local title, _, _, _, _, _, frequency, questID = GetQuestLogTitle(questLogIndex)
 local info = UIDropDownMenu_CreateInfo()

 info.isNotRadio = true
 info.notCheckable = true
 info.isTitle  = true

 -- blank space
 info.text = " "
 UIDropDownMenu_AddButton(info, level)
 -- addon title
 info.text = "|T"..dgcicon..":16|t "..addontitle
 UIDropDownMenu_AddButton(info, level)
 -- quest ID
 info.text = AddColor(L["b_questid"].." "..dropDownFrame.questID, "GOLD")
 UIDropDownMenu_AddButton(info, level)
 -- add buttons
 info.isTitle  = false
 info.disabled = false
 info.text = L["add_quest"]
 info.func = function()

  if DailyGlobalCheck:getQData(questID) then
   print(L["bf_err2"])
   return
  end

  local data = {}
  local map = GetQuestWorldMapAreaID(questID)
  data[ZONE] = map and GetMapNameByID(map) or ""
  data[NAME] = title
  data[PREF] = ""
  data[SUFF] = ""
  data[QUESTTYPE] = frequency == 1 and "Q" or frequency == LE_QUEST_FREQUENCY_WEEKLY and "W" or nil
  DGC_CustomLists.QuestsData[questID] = data
  print(string.format(L["questlog_quest_added"], AddColor(title, "NAME_COLOR")))
 end
 UIDropDownMenu_AddButton(info, level)

 addCategory = questCategory
 
 --DEFAULT_CHAT_FRAME:AddMessage("Category: " .. questCategory)
end

hooksecurefunc("ToggleDropDownMenu", hookToggleDropDownMenu)

-- slash command
SLASH_DAILYGLOBALCHECK1 = "/dgcheck"
SLASH_DAILYGLOBALCHECK2 = "/dgc"
SlashCmdList["DAILYGLOBALCHECK"] = DGCInit